package com.fanruan;

import com.fanruan.handler.DispatcherImpl;
import com.fanruan.pojo.message.RpcRequest;
import com.fanruan.serializer.KryoSerializer;
import com.fanruan.serializer.Serializer;
import io.socket.client.IO;
import io.socket.client.Socket;
import io.socket.engineio.client.transports.WebSocket;
import okhttp3.Dispatcher;
import okhttp3.OkHttpClient;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.TimeUnit;


/**
 * @author YiChen Dai
 */
public class AgentStarter {

    protected static final Logger logger = LogManager.getLogger();

    public final static Serializer SERIALIZER = new KryoSerializer();

    public static DispatcherImpl dispatcherImpl;

    public static String AgentID;

    public AgentStarter(String[][] DBs) {
        dispatcherImpl = new DispatcherImpl();
        try {
            createSocket(DBs);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void createSocket(String[][] DBs){
        logger.debug("加载配置");
        IO.Options options = new IO.Options();
        try{
            InputStream in = this.getClass().getResourceAsStream("/socket.properties");
            Properties props = new Properties();

            InputStreamReader inputStreamReader = new InputStreamReader(in, StandardCharsets.UTF_8);
            props.load(inputStreamReader);

            options.transports = new String[]{WebSocket.NAME};
            options.reconnectionAttempts = Integer.parseInt(props.getProperty("reconnectionAttempts"));
            options.query = "agentID=" + props.getProperty("agentID");
            AgentID = props.getProperty("agentID");
            options.reconnectionDelay = Integer.parseInt(props.getProperty("reconnectionDelay"));
            options.timeout = Integer.parseInt(props.getProperty("timeout"));
            String uri = props.getProperty("uri");
            in.close();

            // config the max number of socket
            int MAX_CLIENTS = 10;
            Dispatcher dispatcher = new Dispatcher();
            dispatcher.setMaxRequests(MAX_CLIENTS * 2);
            dispatcher.setMaxRequestsPerHost(MAX_CLIENTS * 2);

            OkHttpClient okHttpClient = new OkHttpClient.Builder()
                    .dispatcher(dispatcher)
                    // important for HTTP long-polling
                    .readTimeout(1, TimeUnit.MINUTES)
                    .build();

            options.callFactory = okHttpClient;
            options.webSocketFactory = okHttpClient;

            Socket defaultSocket = IO.socket(URI.create(uri), options);
            DispatcherImpl.CACHE.registerSocket("/", defaultSocket);
            configDefaultSocket(defaultSocket);

            for(String[] dbInfo : DBs){
                Socket socket = IO.socket(URI.create(uri + "/" + dbInfo[0]), options);
                DispatcherImpl.CACHE.registerSocket(dbInfo[0], socket);
                configSocket(socket, dbInfo[0]);
                Class.forName(dbInfo[1]);
                socket.connect();
            }

        }catch (Exception e){
            e.printStackTrace();
        }
    }

    public void shutDown(){
        List<Socket> sockets = DispatcherImpl.CACHE.getSockets();
        for(Socket socket : sockets){
            socket.close();
        }
    }

    private void configDefaultSocket(Socket socket){
        socket.on(Socket.EVENT_CONNECT, (objects) ->
            logger.info("default-socket connected!")
        );

        socket.on(Socket.EVENT_CONNECT_ERROR, objects ->
            logger.info("default-socket error: " +  objects[0].toString())
        );

        socket.on(Socket.EVENT_DISCONNECT, objects -> {
            for(Object obj : objects){
                logger.info("default-socket closed: " + obj.toString());
            }
        });
    }

    private void configSocket(Socket socket, String dbName){
        socket.on(Socket.EVENT_CONNECT, objects ->
            logger.info(dbName + "-socket connected!")
        );

        socket.on(Socket.EVENT_DISCONNECT, objects -> {
            for(Object obj : objects){
                logger.info(dbName + "-socket closed: " + obj.toString());
            }
        });

        socket.on(Socket.EVENT_CONNECT_ERROR, objects ->
            logger.info(dbName + "-socket error: " +  objects[0].toString())
        );

        socket.on("RPCRequest", objects -> {
            RpcRequest rpcRequest = SERIALIZER.deserialize((byte[]) objects[0], RpcRequest.class);
            logger.debug(dbName + "-RPCRequest: " + rpcRequest.toString());
            try {
                dispatcherImpl.doDispatch(rpcRequest, dbName);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }
}
