package com.fanruan.handler;


import com.fanruan.agent.jdbc.Parametrical;
import com.fanruan.annotation.BindClass;
import com.fanruan.annotation.BindingParamter;
import com.fanruan.annotation.WithBindingParameter;
import com.fanruan.cache.BeanCacheImpl;

import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

/**
 * Some utils for MyDispatcher
 * @author Yichen Dai
 */
public class DispatcherHelper {

    private static final Map<String, Class<?>> WRAPPER_CLASS_MAP = new HashMap<String, Class<?>>(){
        {
            put("Integer", Integer.TYPE);
            put("Short", Short.TYPE);
            put("Long", Long.TYPE);
            put("Double", Double.TYPE);
            put("Float", Float.TYPE);
            put("Byte", Byte.TYPE);
            put("Character", Character.TYPE);
            put("Boolean", Boolean.TYPE);
        }
    };

    public static boolean isInCacheList(Object obj){
        if(obj == null) {
            return false;
        }
        Class<?> clazz = obj.getClass();
        return clazz.isAnnotationPresent(BindClass.class);
    }

    public static void replaceBindingParameter(BeanCacheImpl beanCache, Method method, Object[] args) throws SQLException {
        Parameter[] pars = method.getParameters();
        for(int i=0; i<pars.length; i++){
            Parameter par = pars[i];
            if(par.isAnnotationPresent(BindingParamter.class)){
                Parametrical obj  = (Parametrical) beanCache.getCachedInstances((String)args[i], par.getType());
                args[i] = obj.getHandler();
            }
        }
    }

    public static boolean isWraps(Class<?> clz){
        if(clz == null) {
            return false;
        }
        return WRAPPER_CLASS_MAP.containsKey(getClassName(clz.getName()));
    }

    public static Class<?> castToPrimitiveClass(Class<?> clz){
        return WRAPPER_CLASS_MAP.get(getClassName(clz.getName()));
    }

    public static String getClassName(String fullyQualifiedClassName){
        String[] arr = fullyQualifiedClassName.trim().split("\\.");
        int n = arr.length;
        if(n < 1) {
            throw new RuntimeException("the class's name invoked is wrong");
        }
        return arr[n-1];
    }

    public static boolean isWithBindingParameter(Method method){
        return method.isAnnotationPresent(WithBindingParameter.class);
    }
}
