package com.fanruan.handler;


import com.fanruan.cache.BeanCacheImpl;
import com.fanruan.pojo.message.RpcRequest;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;


/**
 * @author Yichen Dai
 */
public class DispatcherImpl implements Dispatcher{
    protected static final Logger logger = LogManager.getLogger();

    public DispatcherImpl(){}

    @Override
    public void doDispatch(RpcRequest rpcRequest, String dbName) {
        logger.debug("do dispatcher");
        BeanCacheImpl beanCache = CACHE.getBeanCache(dbName);

        Object res = null;
        try {
            res = invokeAsRequest(rpcRequest, beanCache);
        }catch (Throwable t){
            RESPONSE_EMITTER_IMPL.sendError(CACHE.getSocket(dbName), rpcRequest, t);
        }

        if(res != null && !DispatcherHelper.isInCacheList(res)){
            RESPONSE_EMITTER_IMPL.replyWithData(CACHE.getSocket(dbName), rpcRequest, res);
        }else {
            RESPONSE_EMITTER_IMPL.sendOk(CACHE.getSocket(dbName), rpcRequest);
        }
    }

    @Override
    public Object invokeAsRequest(RpcRequest rpcRequest, BeanCacheImpl beanCache) throws Throwable {
        String fullName = rpcRequest.getServiceClassName();
        Class<?> clazz = Class.forName(fullName);
        String methodName = rpcRequest.getMethodName();
        Object[] args = rpcRequest.getArgs();
        Class<?>[] argTypes = rpcRequest.getArgTypes();
        Object calledClassInstance = null;
        // The ID of the rpcRequest could be save as the ID of an instance
        // Because one instance can only been create just once for an unique rpcRequest
        String IDToCache = rpcRequest.getID();
        String IDToInvoke = rpcRequest.getIDToInvoke();

        String className = DispatcherHelper.getClassName(fullName);

        // If BeanCache contains instance, get it; if not, create it.
        if(IDToInvoke == null){
            try{
                // create
                calledClassInstance = Class.forName(fullName).newInstance();
            }catch (Exception e) {
                e.printStackTrace();
            }
                beanCache.cacheInstance(IDToCache, calledClassInstance);
        }else{
            calledClassInstance = beanCache.getCachedInstances(IDToInvoke, clazz);
        }

        Method method;

        try {
            // The primitive variable's type will be automatically packaged when passed as a class object,
            // And an error will be reported when the method with the primitive variable as the parameter is called
            method = clazz.getDeclaredMethod(methodName, argTypes);
        }catch (Exception e){
            for(int i=0; i<argTypes.length; i++){
                Class<?> clz = argTypes[i];
                if(DispatcherHelper.isWraps(clz)){
                    argTypes[i] = DispatcherHelper.castToPrimitiveClass(clz);
                }
            }
            method = clazz.getDeclaredMethod(methodName, argTypes);
        }

        if (DispatcherHelper.isWithBindingParameter(method)){
            DispatcherHelper.replaceBindingParameter(beanCache, method, args);
        }

        Object res = null;

        try{
            res = method.invoke(calledClassInstance, args);
        }catch (InvocationTargetException e){
            throw e.getCause();
        }

        // Cached some instances need to be invoke later.
        // Some method return null

        if(DispatcherHelper.isInCacheList(res)) {
            beanCache.cacheInstance(rpcRequest.getID(), res);
        }

        logger.debug("invoke" + className + "-" + methodName + " and return value : " + res);

        return res;
    }


}
