import com.fanruan.utils.DBProperties;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.sql.*;

/**
 * @author Yichen Dai
 * @date 2022/8/17 11:25
 */
public class HSQLTest {

    @BeforeEach
    void startHSQL() {
        try {
            Class.forName(DBProperties.HSQL[1]);
            Thread.sleep(1000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    void getConnection() throws SQLException {
        Connection conn = null;
        try {
            conn = DriverManager.getConnection("jdbc:hsqldb:mem:test;sql.syntax_mys=true", "sa", "");
            Assertions.assertNotNull(conn, "can't get connection");
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            conn.close();
        }
    }

    @Test
    public void testPrepareDDL() throws SQLException {
        Connection connection = DriverManager.getConnection("jdbc:hsqldb:mem:test;sql.syntax_mys=true", "sa", "");
        try {
            try (PreparedStatement prepStatement =
                         connection.prepareStatement("CREATE TABLE city (id INTEGER, name varchar(20)); ")) {
                prepStatement.execute();
            }

            try (ResultSet resultSet =
                         connection.createStatement().executeQuery("SELECT \n" +
                                 "TABLE_NAME\n" +
                                 "FROM INFORMATION_SCHEMA.TABLES\n" +
                                 "where TABLE_NAME like 'CITY';")) {
                // result should only have one row since table is created
                while(resultSet.next()){
                    System.out.println(resultSet.getString(1));
                }
            }
        } finally {
            connection.createStatement().execute("drop table if exists city");
        }
    }




    private static String createStoredProcedure ="CREATE PROCEDURE get_city_name(" +
            "IN c_id INT, " +
            "OUT city_name varchar(20)) " +
            "READS SQL DATA " +
            "BEGIN ATOMIC " +
            "   SELECT city.name INTO city_name FROM city WHERE id = c_id; " +
            "END;";

    @Test
    public void test() throws SQLException {
        Connection conn = DriverManager.getConnection("jdbc:hsqldb:mem:test;sql.syntax_mys=true", "sa", "");
        Statement statement = conn.createStatement();
        statement.execute("create table TestTable " +
                "   (" +
                "   testString varchar(20), " +
                "   testBoolean BOOLEAN, " +
                "   testByte NUMERIC(10), " +
                "   testShort NUMERIC(10), " +
                "   testInt INTEGER, " +
                "   testLong BIGINT, " +
                "   testFloat FLOAT, " +
                "   testDouble DOUBLE, " +
                "   testDecimal DECIMAL);"
        );
    }

    /**
     * test to create statement instance and prepareStatement, create table, select form table, delete from table
     */
    @Test
    void testCURD(){
        Connection conn = null;
        Statement st = null;
        PreparedStatement pst = null;
        ResultSet rs = null;
        try {
            conn = DriverManager.getConnection("jdbc:hsqldb:mem:test;sql.syntax_mys=true", "sa", "");
            st = conn.createStatement();
            st.executeUpdate("DROP TABLE student IF EXISTS;");

            st.executeUpdate("CREATE TABLE student (" +
                    "student_id INTEGER GENERATED BY DEFAULT AS IDENTITY " +
                    "(START WITH 1, INCREMENT BY 1) NOT NULL," +
                    "student_name VARCHAR(100) NOT NULL," +
                    "student_address VARCHAR(100) NOT NULL," +
                    "PRIMARY KEY (student_id)" +
                    ");");

            st.executeUpdate("INSERT INTO student VALUES" +
                    "(1, '张三', '上海')," +
                    "(2, '李四', '北京')," +
                    "(3, '王五', '成都');");




            pst = conn.prepareStatement("delete from student where student_id = ?");

            pst.setInt(1, 1);

            pst.executeUpdate();

            rs = st.executeQuery("select * from student");

            String[] nameStrings = new String[]{"张三", "李四", "王五"};
            String[] addressStrings = new String[]{"上海", "北京", "成都"};

            int num = 2;
            while(rs.next()) {
                Assertions.assertEquals(rs.getInt("student_id"), num);
                Assertions.assertEquals(rs.getString("student_name"), nameStrings[num-1]);
                Assertions.assertEquals(rs.getString("student_address"), addressStrings[num-1]);
                num++;
            }


        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try{
                rs.close();
                st.close();
                pst.close();
                conn.close();
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }


}
