package com.fanruan.handler;

import com.fanruan.agent.jdbc.driver.AgentDriver;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;



/**
 * @author Yichen Dai
 * @date 2022/8/22 15:07
 */
class DispatcherHelperTest{



    @Test
    void isInCacheList() {
        Assertions.assertTrue(DispatcherHelper.isInCacheList(new AgentDriver()));

        Assertions.assertTrue(DispatcherHelper.isInCacheList("com.fanruan.agent.jdbc.agentDriver"));
        Assertions.assertTrue(DispatcherHelper.isInCacheList("com.fanruan.agent.jdbc.agentConnection"));
        Assertions.assertTrue(DispatcherHelper.isInCacheList("com.fanruan.agent.jdbc.agentStatement"));
        Assertions.assertTrue(DispatcherHelper.isInCacheList("com.fanruan.agent.jdbc.agentPrepareStatement"));
        Assertions.assertTrue(DispatcherHelper.isInCacheList("com.fanruan.agent.jdbc.agentResultSet"));

        Assertions.assertFalse(DispatcherHelper.isInCacheList("java.lang.String"));
        Assertions.assertFalse(DispatcherHelper.isInCacheList("java.lang.Integer"));
        Assertions.assertFalse(DispatcherHelper.isInCacheList("com.fanruan.agent.cache.BeanCache"));
        Assertions.assertFalse(DispatcherHelper.isInCacheList(""));
        Assertions.assertFalse(DispatcherHelper.isInCacheList(null));

    }

    @Test
    void isWraps() {
        Assertions.assertTrue(DispatcherHelper.isWraps(java.lang.Byte.class));
        Assertions.assertTrue(DispatcherHelper.isWraps(java.lang.Character.class));
        Assertions.assertTrue(DispatcherHelper.isWraps(java.lang.Short.class));
        Assertions.assertTrue(DispatcherHelper.isWraps(java.lang.Integer.class));
        Assertions.assertTrue(DispatcherHelper.isWraps(java.lang.Long.class));
        Assertions.assertTrue(DispatcherHelper.isWraps(java.lang.Boolean.class));
        Assertions.assertTrue(DispatcherHelper.isWraps(java.lang.Float.class));
        Assertions.assertTrue(DispatcherHelper.isWraps(java.lang.Double.class));

        Assertions.assertFalse(DispatcherHelper.isWraps(Integer.TYPE));
        Assertions.assertFalse(DispatcherHelper.isWraps(java.lang.String.class));
        Assertions.assertFalse(DispatcherHelper.isWraps(null));
    }

    @Test
    void castToPrimitiveClass() {
        Assertions.assertEquals(Byte.TYPE, DispatcherHelper.castToPrimitiveClass(Byte.class));
        Assertions.assertEquals(Character.TYPE, DispatcherHelper.castToPrimitiveClass(Character.class));
        Assertions.assertEquals(Short.TYPE, DispatcherHelper.castToPrimitiveClass(Short.class));
        Assertions.assertEquals(Integer.TYPE, DispatcherHelper.castToPrimitiveClass(Integer.class));
        Assertions.assertEquals(Long.TYPE, DispatcherHelper.castToPrimitiveClass(Long.class));
        Assertions.assertEquals(Boolean.TYPE, DispatcherHelper.castToPrimitiveClass(Boolean.class));
        Assertions.assertEquals(Float.TYPE, DispatcherHelper.castToPrimitiveClass(Float.class));
        Assertions.assertEquals(Double.TYPE, DispatcherHelper.castToPrimitiveClass(Double.class));
    }

    @Test
    void getClassName() {
        Assertions.assertEquals("agentDriver", DispatcherHelper.getClassName("com.fanruan.agent.jdbc.agentDriver"));
        Assertions.assertEquals("agentDriver", DispatcherHelper.getClassName("  com.fanruan.agent.jdbc.agentDriver  "));
        Assertions.assertEquals("fakeName", DispatcherHelper.getClassName("fakeName"));
        Assertions.assertEquals("", DispatcherHelper.getClassName(""));
        Assertions.assertThrows(RuntimeException.class, () -> DispatcherHelper.getClassName(null));
    }
}