package com.fanruan.service.jdbc.connection;


import com.fanruan.annotation.*;
import com.fanruan.service.jdbc.*;
import com.fanruan.service.jdbc.statement.ServiceCallableStatement;
import com.fanruan.service.jdbc.statement.ServicePreparedStatement;
import com.fanruan.service.jdbc.statement.ServiceStatement;
import com.fanruan.proxy.ProxyFactory;
import com.fanruan.exception.NotImplementedException;

import java.sql.*;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.Executor;

/**
 * @author Yichen Dai
 */
@RemoteClass(remoteClassName = "com.fanruan.agent.jdbc.connection.AgentConnection")
public class ServiceConnection extends BasedBind implements Connection {

    public ServiceConnection(){
    }

    @Override
    public Statement createStatement(){
        ServiceStatement st = (ServiceStatement) ProxyFactory.getProxy(ServiceStatement.class, info);
        st.setInfo(info);
        st.setConnection(this);
        return st;
    }

    @Override
    public PreparedStatement prepareStatement(String sql) throws SQLException {
        ServicePreparedStatement pst = (ServicePreparedStatement) ProxyFactory.getProxy(ServicePreparedStatement.class, info);
        pst.setInfo(info);
        return pst;
    }

    @Override
    public CallableStatement prepareCall(String sql) throws SQLException {
        ServiceCallableStatement cst = (ServiceCallableStatement) ProxyFactory.getProxy(ServiceCallableStatement.class, info);
        cst.setInfo(info);
        cst.setConnection(this);
        return cst;
    }

    @Override
    public String nativeSQL(String sql) throws SQLException {
        return sql;
    }

    @Override
    public void setAutoCommit(boolean autoCommit) throws SQLException {
    }

    @Override
    public boolean getAutoCommit() throws SQLException {
        return true;
    }

    @Override
    public void commit() throws SQLException {

    }
    @Override
    public void rollback() throws SQLException {
    }

    @Override
    public void close() throws SQLException {
    }

    @Override
    public boolean isClosed() throws SQLException {
        return true;
    }

    @Override
    public DatabaseMetaData getMetaData() throws SQLException {
        ServiceDatabaseMetaData metaData = (ServiceDatabaseMetaData) ProxyFactory.getProxy(ServiceDatabaseMetaData.class, info);
        metaData.setConn(this);
        metaData.setInfo(info);
        return metaData;
    }

    @Override
    public void setReadOnly(boolean readOnly) throws SQLException {

    }

    @Override
    public boolean isReadOnly() throws SQLException {
        return false;
    }

    @Override
    public void setCatalog(String catalog) throws SQLException {

    }

    @Override
    public String getCatalog() throws SQLException {
        return null;
    }

    @Override
    public void setTransactionIsolation(int level) throws SQLException {

    }

    @Override
    public int getTransactionIsolation() throws SQLException {
        return 0;
    }

    @Override
    public SQLWarning getWarnings() throws SQLException {
        return null;
    }

    @Override
    public void clearWarnings() throws SQLException {

    }

    @Override
    public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
        ServiceStatement st = (ServiceStatement) ProxyFactory.getProxy(ServiceStatement.class, info);
        st.setInfo(info);
        st.setConnection(this);
        return st;
    }

    @Override
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        ServicePreparedStatement pst = (ServicePreparedStatement) ProxyFactory.getProxy(ServicePreparedStatement.class, info);
        pst.setInfo(info);
        pst.setConnection(this);
        return pst;
    }

    @Override
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        ServiceCallableStatement cst = (ServiceCallableStatement) ProxyFactory.getProxy(ServiceCallableStatement.class, info);
        cst.setInfo(info);
        cst.setConnection(this);
        return cst;
    }

    @Override
    public Map<String, Class<?>> getTypeMap() throws SQLException {
        return null;
    }

    @Override
    public void setTypeMap(Map<String, Class<?>> map) throws SQLException {

    }

    @Override
    public void setHoldability(int holdability) throws SQLException {

    }

    @Override
    public int getHoldability() throws SQLException {
        return 0;
    }

    @Override
    public Savepoint setSavepoint() throws SQLException {
        return (Savepoint) ProxyFactory.getProxy(ServiceSavepoint.class, info);
    }

    @Override
    public Savepoint setSavepoint(String name) throws SQLException {
        return (Savepoint) ProxyFactory.getProxy(ServiceSavepoint.class, info);
    }

    @Override
    @WithBindingParameter
    public void rollback(@BindingParamter Savepoint savepoint) throws SQLException {

    }

    @Override
    @WithBindingParameter
    public void releaseSavepoint(@BindingParamter Savepoint savepoint) throws SQLException {
    }

    @Override
    public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        ServiceStatement st = (ServiceStatement) ProxyFactory.getProxy(ServiceStatement.class, info);
        st.setInfo(info);
        st.setConnection(this);
        return st;
    }

    @Override
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        ServicePreparedStatement pst = (ServicePreparedStatement) ProxyFactory.getProxy(ServicePreparedStatement.class, info);
        pst.setInfo(info);
        pst.setConnection(this);
        return pst;
    }

    @Override
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        ServiceCallableStatement cst = (ServiceCallableStatement) ProxyFactory.getProxy(ServiceCallableStatement.class, info);
        cst.setInfo(info);
        cst.setConnection(this);
        return cst;
    }

    @Override
    public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
        ServicePreparedStatement pst = (ServicePreparedStatement) ProxyFactory.getProxy(ServicePreparedStatement.class, info);
        pst.setInfo(info);
        pst.setConnection(this);
        return pst;
    }

    @Override
    public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
        ServicePreparedStatement pst = (ServicePreparedStatement) ProxyFactory.getProxy(ServicePreparedStatement.class, info);
        pst.setInfo(info);
        pst.setConnection(this);
        return pst;
    }

    @Override
    public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
        ServicePreparedStatement pst = (ServicePreparedStatement) ProxyFactory.getProxy(ServicePreparedStatement.class, info);
        pst.setInfo(info);
        pst.setConnection(this);
        return pst;
    }

    @Override
    public Clob createClob() throws SQLException {
        ServiceClob clob = (ServiceClob) ProxyFactory.getProxy(ServiceClob.class, info);
        clob.setInfo(info);
        return clob;
    }

    @Override
    public Blob createBlob() throws SQLException {
        ServiceBlob blob = (ServiceBlob) ProxyFactory.getProxy(ServiceBlob.class, info);
        blob.setInfo(info);
        return blob;
    }

    @Override
    public NClob createNClob() throws SQLException {
        return (NClob) ProxyFactory.getProxy(ServiceNClob.class, info);
    }

    @Override
    @NotImplemented
    @LocalMethod
    public SQLXML createSQLXML() throws SQLException {
        throw new SQLException("Not Implemented");
    }

    @Override
    public boolean isValid(int timeout) throws SQLException {
        return false;
    }

    @Override
    public void setClientInfo(String name, String value) throws SQLClientInfoException {

    }

    @Override
    public void setClientInfo(Properties properties) throws SQLClientInfoException {

    }

    @Override
    public String getClientInfo(String name) throws SQLException {
        return null;
    }

    @Override
    public Properties getClientInfo() throws SQLException {
        return null;
    }

    @Override
    public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
        ServiceArray array = (ServiceArray) ProxyFactory.getProxy(ServiceArray.class, info);
        array.setInfo(info);
        return array;
    }

    @Override
    public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
        ServiceStruct struct = (ServiceStruct) ProxyFactory.getProxy(ServiceStruct.class, info);
        struct.setInfo(info);
        return struct;
    }

    @Override
    public void setSchema(String schema) throws SQLException {

    }

    @Override
    public String getSchema() throws SQLException {
        return null;
    }

    @Override
    public void abort(Executor executor) throws SQLException {

    }

    @Override
    public void setNetworkTimeout(Executor executor, int milliseconds) throws SQLException {

    }

    @Override
    public int getNetworkTimeout() throws SQLException {
        return 0;
    }

    @Override
    @NotImplemented
    @LocalMethod
    public <T> T unwrap(Class<T> iface) throws SQLException {
        throw new SQLException("Not Implemented");
    }

    @Override
    public boolean isWrapperFor(Class<?> iface) throws NotImplementedException {
        return true;
    }
}
