package com.fanruan.service.jdbc.driver;

import com.fanruan.annotation.LocalMethod;
import com.fanruan.annotation.NotImplemented;
import com.fanruan.annotation.RemoteClass;
import com.fanruan.service.jdbc.AbstractBind;
import com.fanruan.service.jdbc.BasedBind;
import com.fanruan.service.jdbc.connection.ServiceConnection;
import com.fanruan.proxy.ProxyFactory;

import java.sql.*;
import java.util.Properties;
import java.util.logging.Logger;


/**
 * @author Yichen Dai
 */
@RemoteClass(remoteClassName = "com.fanruan.agent.jdbc.driver.AgentDriver")
public class ServiceDriver extends BasedBind implements Driver {

    static public final int DRIVER_VERSION_MAJOR = 1;
    static public final int DRIVER_VERSION_MINOR = 1;

    static{
        try {
            DriverManager.registerDriver((ServiceDriver) ProxyFactory.getProxy(ServiceDriver.class, null));
        } catch (Exception e) {
            throw new RuntimeException("Can't register driver");
        }
    }


    @Override
    public Connection connect(String url, Properties info){
        String dbName = info.getProperty("agentDBName");
        if(dbName == null){
            dbName = url.split(":")[1];
            info.setProperty("agentDBName", dbName);
        }
        ServiceConnection myConn = (ServiceConnection) ProxyFactory.getProxy(ServiceConnection.class, info);
        myConn.setInfo(info);
        return myConn;
    }

    @Override
    @LocalMethod
    public boolean acceptsURL(String url){
        return true;
    }

    @Override
    @LocalMethod
    public DriverPropertyInfo[] getPropertyInfo(String url, Properties info){
        return new DriverPropertyInfo[0];
    }

    @Override
    @LocalMethod
    public int getMajorVersion() {
        return DRIVER_VERSION_MAJOR;
    }

    @Override
    @LocalMethod
    public int getMinorVersion() {
        return DRIVER_VERSION_MINOR;
    }

    @Override
    @LocalMethod
    public boolean jdbcCompliant() {
        return false;
    }

    @Override
    @LocalMethod
    @NotImplemented
    public Logger getParentLogger() throws SQLFeatureNotSupportedException {
        throw new SQLFeatureNotSupportedException("Not Implemented");
    }
}
