import com.corundumstudio.socketio.SocketIOClient;
import com.fanruan.cache.ClientCache;
import com.fanruan.utils.DBProperties;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;


import java.sql.*;
import java.util.Properties;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;

/**
 * @author Yichen Dai
 * @date 2022/8/18 9:49
 */
public class ServiceTest {

    @BeforeEach
    void listen() throws ExecutionException, InterruptedException {
        FutureTask<SocketIOClient> ft = new FutureTask<>(
                () -> {
                    while(ClientCache.getClient("1001", DBProperties.HSQL[0]) == null){
                        try {
                            Thread.sleep(100);
                        }catch (Exception e){
                            e.printStackTrace();
                        }
                    }
                    return ClientCache.getClient("1001", DBProperties.HSQL[0]);
                }
        );
        ft.run();
        ft.get();
    }

    @Test
    void testCURD(){
        Properties info = new Properties();
        info.setProperty("user", "sa");
        info.setProperty("password", "");
        info.setProperty("agentID", "1001");
        info.setProperty("agentDBName", DBProperties.HSQL[0]);

        Connection conn = null;
        Statement st = null;
        PreparedStatement pst = null;
        ResultSet rs = null;
        try {

            // 创建 连接
            conn = DriverManager.getConnection("jdbc:hsqldb:mem:test", info);

            // 创建 statement
            st = conn.createStatement();

            // 创建表
            st.executeUpdate("DROP TABLE student IF EXISTS;");

            st.executeUpdate("CREATE TABLE student (" +
                    "student_id INTEGER GENERATED BY DEFAULT AS IDENTITY " +
                    "(START WITH 1, INCREMENT BY 1) NOT NULL," +
                    "student_name VARCHAR(100) NOT NULL," +
                    "student_address VARCHAR(100) NOT NULL," +
                    "PRIMARY KEY (student_id)" +
                    ");");

            // 插入数据
            st.executeUpdate("INSERT INTO student VALUES" +
                    "(1, '张三', '上海')," +
                    "(2, '李四', '北京')," +
                    "(3, '王五', '成都');");



            // 预查询语句 删除指定 ID
            pst = conn.prepareStatement("delete from student where student_id = ?");

            pst.setInt(1, 1);

            pst.executeUpdate();

            rs = st.executeQuery("select * from student");

            String[] nameStrings = new String[]{"张三", "李四", "王五"};
            String[] addressStrings = new String[]{"上海", "北京", "成都"};

            // 结果集断言
            int num = 2;
            while(rs.next()) {
                Assertions.assertEquals(rs.getInt("student_id"), num);
                Assertions.assertEquals(rs.getString("student_name"), nameStrings[num-1]);
                Assertions.assertEquals(rs.getString("student_address"), addressStrings[num-1]);
                num++;
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try{
                rs.close();
                st.close();
                pst.close();
                conn.close();
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }
}
