package com.fanruan;

import com.fanruan.annotation.RemoteClass;
import com.fanruan.pojo.message.RpcRequest;
import com.fanruan.service.jdbc.AbstractBind;
import com.fanruan.utils.Commons;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import java.io.*;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Yichen Dai
 * @date 2022/8/23 15:47
 */


public class BaseJDBCTest extends AbstractDriverTest{
    final String dbName = "hsql";
    final Map<String, RpcRequest> map = new HashMap(1);

    int getSizeOfResultSet(ResultSet rs) throws SQLException {
        int count = 0;
        while (rs.next()) {
            count++;
        }
        return count;
    }

    List<String> getInfoBySQL(String sqlCmd) throws SQLException {
        Connection con = getConnection();
        Statement st = con.createStatement();
        List<String> result = new ArrayList<>();
        ResultSet rs = st.executeQuery(sqlCmd);
        while (rs.next()) {
            result.add(rs.getString(1));
        }
        return result;
    }

    public RpcRequest createRequest(AbstractBind bind, String methodName, Object[] args, Class<?>[] argTypes) {
        RpcRequest request = new RpcRequest();

        request.setID(Commons.getID())
                .setIDToInvoke(bind.getID())
                .setMethodName(methodName)
                .setArgs(args)
                .setArgTypes(argTypes);

        Class clazz = bind.getClass();

        if(clazz.isAnnotationPresent(RemoteClass.class)){
            RemoteClass annotation = (RemoteClass) clazz.getAnnotation(RemoteClass.class);
            request.setServiceClassName(annotation.remoteClassName());
        }else{
            request.setServiceClassName(clazz.getName());
        }
        return request;
    }

    public boolean isEqualRequest(RpcRequest r1, RpcRequest r2){
        String[] s1 = new String[]{
                r1.getID(),
                r1.getIDToInvoke(),
                r1.getMethodName(),
                r1.getServiceClassName()
        };

        String[] s2 = new String[]{
                r2.getID(),
                r2.getIDToInvoke(),
                r2.getMethodName(),
                r2.getServiceClassName()
        };

        if(isNotEqualStringArray(s1, s2)){
            return false;
        }

        Object[] o1 = r1.getArgs();
        Object[] o2 = r2.getArgs();
        int o1_len = o1.length;
        int o2_len = o2.length;
        String[] o1_sa = new String[o1_len];
        String[] o2_sa = new String[o2_len];

        for(int i=0; i<o1_len; i++){
            o1_sa[i] = o1[i].getClass().getName();
            o2_sa[i] = o2[i].getClass().getName();
        }

        if(isNotEqualStringArray(o1_sa, o2_sa)){
            return false;
        }

        Class<?>[] c1 = r1.getArgTypes();
        Class<?>[] c2 = r2.getArgTypes();
        int c1_len = c1.length;
        int c2_len = c2.length;
        String[] c1_sa = new String[c1_len];
        String[] c2_sa = new String[c2_len];

        for(int i=0; i<o1_len; i++){
            c1_sa[i] = c1[i].getName();
            c2_sa[i] = c2[i].getName();
        }

        if(isNotEqualStringArray(c1_sa, c2_sa)){
            return false;
        }

        return true;
    }

    public boolean isNotEqualStringArray(String[] s1, String[] s2){
        if(s1.length != s2.length){
            return true;
        }

        for(int i=0; i<s1.length; i++){
            if(s1[i] == null && s2[i] != null){
                return true;
            }else if(s1[i] != null && !s1[i].equals(s2[i])){
                return true;
            }
        }

        return false;
    }



    public static void closeSQLObjects(Object...objects) throws SQLException {
        for(Object obj : objects){
            if(obj != null){
                if(obj instanceof Connection){
                    Connection conn = (Connection) obj;
                    conn.close();
                }else if(obj instanceof Statement){
                    Statement statement = (Statement) obj;
                    statement.close();
                }else if(obj instanceof PreparedStatement){
                    PreparedStatement preparedStatement = (PreparedStatement) obj;
                    preparedStatement.close();
                }else if(obj instanceof ResultSet){
                    ResultSet resultSet = (ResultSet) obj;
                    resultSet.close();
                }else{
                    throw new SQLException("Some arguments passed in are not closable.");
                }
            }
        }
    }
}

class FakeInputStream extends InputStream {
    @Override
    public int read() throws IOException {
        return -1;
    }
}

class FakeReader extends Reader {
    @Override
    public int read(char[] cbuf, int off, int len) throws IOException {
        return 0;
    }

    @Override
    public void close() throws IOException {}
}

class FakeSQLXML implements SQLXML {
    @Override
    public void free() throws SQLException {}

    @Override
    public InputStream getBinaryStream() throws SQLException {
        return null;
    }

    @Override
    public OutputStream setBinaryStream() throws SQLException {
        return null;
    }

    @Override
    public Reader getCharacterStream() throws SQLException {
        return null;
    }

    @Override
    public Writer setCharacterStream() throws SQLException {
        return null;
    }

    @Override
    public String getString() throws SQLException {
        return null;
    }

    @Override
    public void setString(String value) throws SQLException {}

    @Override
    public <T extends Source> T getSource(Class<T> sourceClass) throws SQLException {
        return null;
    }

    @Override
    public <T extends Result> T setResult(Class<T> resultClass) throws SQLException {
        return null;
    }
}

class FakeBlob implements Blob {
    @Override
    public long length() throws SQLException {
        return 0;
    }

    @Override
    public byte[] getBytes(long pos, int length) throws SQLException {
        return new byte[0];
    }

    @Override
    public InputStream getBinaryStream() throws SQLException {
        return null;
    }

    @Override
    public long position(byte[] pattern, long start) throws SQLException {
        return 0;
    }

    @Override
    public long position(Blob pattern, long start) throws SQLException {
        return 0;
    }

    @Override
    public int setBytes(long pos, byte[] bytes) throws SQLException {
        return 0;
    }

    @Override
    public int setBytes(long pos, byte[] bytes, int offset, int len) throws SQLException {
        return 0;
    }

    @Override
    public OutputStream setBinaryStream(long pos) throws SQLException {
        return null;
    }

    @Override
    public void truncate(long len) throws SQLException {}

    @Override
    public void free() throws SQLException {}

    @Override
    public InputStream getBinaryStream(long pos, long length) throws SQLException {
        return null;
    }
}

class FakeArray implements Array {
    @Override
    public String getBaseTypeName() throws SQLException {
        return null;
    }

    @Override
    public int getBaseType() throws SQLException {
        return 0;
    }

    @Override
    public Object getArray() throws SQLException {
        return null;
    }

    @Override
    public Object getArray(Map<String, Class<?>> map) throws SQLException {
        return null;
    }

    @Override
    public Object getArray(long index, int count) throws SQLException {
        return null;
    }

    @Override
    public Object getArray(long index, int count, Map<String, Class<?>> map) throws SQLException {
        return null;
    }

    @Override
    public ResultSet getResultSet() throws SQLException {
        return null;
    }

    @Override
    public ResultSet getResultSet(Map<String, Class<?>> map) throws SQLException {
        return null;
    }

    @Override
    public ResultSet getResultSet(long index, int count) throws SQLException {
        return null;
    }

    @Override
    public ResultSet getResultSet(long index, int count, Map<String, Class<?>> map)
            throws SQLException {
        return null;
    }

    @Override
    public void free() throws SQLException {}
}

class FakeRowId implements RowId {
    @Override
    public byte[] getBytes() {
        return new byte[0];
    }
}

class FakeNClob implements NClob {
    @Override
    public long length() throws SQLException {
        return 0;
    }

    @Override
    public String getSubString(long pos, int length) throws SQLException {
        return null;
    }

    @Override
    public Reader getCharacterStream() throws SQLException {
        return null;
    }

    @Override
    public InputStream getAsciiStream() throws SQLException {
        return null;
    }

    @Override
    public long position(String searchstr, long start) throws SQLException {
        return 0;
    }

    @Override
    public long position(Clob searchstr, long start) throws SQLException {
        return 0;
    }

    @Override
    public int setString(long pos, String str) throws SQLException {
        return 0;
    }

    @Override
    public int setString(long pos, String str, int offset, int len) throws SQLException {
        return 0;
    }

    @Override
    public OutputStream setAsciiStream(long pos) throws SQLException {
        return null;
    }

    @Override
    public Writer setCharacterStream(long pos) throws SQLException {
        return null;
    }

    @Override
    public void truncate(long len) throws SQLException {}

    @Override
    public void free() throws SQLException {}

    @Override
    public Reader getCharacterStream(long pos, long length) throws SQLException {
        return null;
    }
}

class FakeRef implements Ref {

    @Override
    public String getBaseTypeName() throws SQLException {
        return null;
    }

    @Override
    public Object getObject(Map<String, Class<?>> map) throws SQLException {
        return null;
    }

    @Override
    public Object getObject() throws SQLException {
        return null;
    }

    @Override
    public void setObject(Object value) throws SQLException {}
}

class FakeSavepoint implements Savepoint{

    @Override
    public int getSavepointId() throws SQLException {
        return 0;
    }

    @Override
    public String getSavepointName() throws SQLException {
        return null;
    }
}

class FakeClob implements Clob{

    @Override
    public long length() throws SQLException {
        return 0;
    }

    @Override
    public String getSubString(long pos, int length) throws SQLException {
        return null;
    }

    @Override
    public Reader getCharacterStream() throws SQLException {
        return null;
    }

    @Override
    public InputStream getAsciiStream() throws SQLException {
        return null;
    }

    @Override
    public long position(String searchstr, long start) throws SQLException {
        return 0;
    }

    @Override
    public long position(Clob searchstr, long start) throws SQLException {
        return 0;
    }

    @Override
    public int setString(long pos, String str) throws SQLException {
        return 0;
    }

    @Override
    public int setString(long pos, String str, int offset, int len) throws SQLException {
        return 0;
    }

    @Override
    public OutputStream setAsciiStream(long pos) throws SQLException {
        return null;
    }

    @Override
    public Writer setCharacterStream(long pos) throws SQLException {
        return null;
    }

    @Override
    public void truncate(long len) throws SQLException {

    }

    @Override
    public void free() throws SQLException {

    }

    @Override
    public Reader getCharacterStream(long pos, long length) throws SQLException {
        return null;
    }
}
