package com.fanruan;

import com.fanruan.cache.BeanCacheImpl;
import com.fanruan.handler.DispatcherImpl;
import com.fanruan.pojo.message.RpcRequest;
import com.fanruan.pojo.message.RpcResponse;
import com.fanruan.proxy.interceptor.InterceptorUtils;
import com.fanruan.service.jdbc.ServiceBlob;
import com.fanruan.service.jdbc.statement.ServiceCallableStatement;
import com.fanruan.utils.Commons;
import org.junit.jupiter.api.*;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.*;
import java.sql.Date;
import java.util.*;

/**
 * @author Yichen Dai
 * @date 2022/8/24 10:22
 */

@TestInstance(TestInstance.Lifecycle.PER_CLASS)
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
public class CallableStatementTest extends BaseJDBCTest{

    private Connection conn = null;
    private Statement statement = null;
    private CallableStatement callableStatement = null;
    private Properties properties = null;

    private static String createStoredProcedure = "CREATE PROCEDURE get_city_name(" +
            "IN c_id INT, " +
            "OUT city_name varchar(20)) " +
            "READS SQL DATA " +
            "BEGIN ATOMIC " +
            "   SELECT city.name INTO city_name FROM city WHERE id = c_id; " +
            "END;";

    private static String callProcedure =
            "{call get_city_name(?, ?) }";

    private static String deleteStoredProcedure =
            "DROP PROCEDURE IF EXISTS cnt_num;";

    @BeforeAll
    public void setUp() throws SQLException{
        openSocket();
        conn = getConnection();
        statement = conn.createStatement();
        statement.execute("DROP TABLE city IF EXISTS");
        statement.executeUpdate("CREATE TABLE city (id INTEGER, name varchar(20)); ");
        statement.executeUpdate("INSERT INTO city VALUES (1, '成都'), (2, '上海'); ");
        statement.execute(deleteStoredProcedure);
        statement.execute(createStoredProcedure);
    }

    @Test
    @Order(1)
    public void testPrepareCall() throws SQLException{
        callableStatement = conn.prepareCall(callProcedure);
        ParameterMetaData parameterMetaData = callableStatement.getParameterMetaData();
        int num = parameterMetaData.getParameterCount();
        Assertions.assertEquals(2, num);
        Assertions.assertEquals(Types.INTEGER, parameterMetaData.getParameterType(1));
        Assertions.assertEquals("INTEGER", parameterMetaData.getParameterTypeName(1));

        callableStatement.setInt(1, 2);
        callableStatement.registerOutParameter(2, Types.VARCHAR);
        callableStatement.execute();
        Assertions.assertEquals("上海", callableStatement.getString(2));
    }

    @Test
    @Order(2)
    void testMethods() throws SQLException {
        ServiceCallableStatement cstm = (ServiceCallableStatement) this.callableStatement;
        String IDtoInvoke = cstm.getID();

        DispatcherImpl dispatcher = agent.dispatcherImpl;
        BeanCacheImpl beanCache = dispatcher.CACHE.getBeanCache(dbName);
        ServiceCallableStatement cstm2 = (ServiceCallableStatement) ProxyFactoryIT.getProxy(ServiceCallableStatement.class, map);

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.registerOutParameter(1, Types.INTEGER);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.registerOutParameter(1, Types.INTEGER, 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.registerOutParameter(1, Types.INTEGER, "int");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.registerOutParameter("param_name", Types.INTEGER);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.registerOutParameter("param_name", Types.INTEGER, 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.registerOutParameter("param_name", Types.INTEGER, "int");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );


        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getArray("param_name");
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getArray(1);
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getBigDecimal("param_name");
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getBigDecimal(1);
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getBlob("param_name");
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getBlob(1);
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getBoolean("param_name");
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getBoolean(1);
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );
        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getByte("param_name");
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
            cstm2.getByte(1);
            RpcRequest request = map.get(null);
            request.setIDToInvoke(IDtoInvoke);
            dispatcher.invokeAsRequest(
                    request,
                    beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getCharacterStream("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getCharacterStream(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getClob("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getClob(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getDate("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getDate(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getDouble("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getDouble(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getFloat("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getFloat(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getInt("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getInt(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getLong("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getLong(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getNCharacterStream("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getNCharacterStream(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getNClob("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getNClob(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setNClob("param_name", new FakeNClob());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setNClob("param_name", new FakeReader(), 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setNClob("param_name", new FakeReader());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getNString("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getNString(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );


        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getObject("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getObject(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getObject(1, String.class);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getRef("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getRef(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getRowId("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getRowId(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getShort("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getShort(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getSQLXML("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getSQLXML(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getString("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getString(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getTime("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getTime(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getTimestamp(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getTimestamp("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );


        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getURL("param_name");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getURL(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getString(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBigDecimal("param_name", BigDecimal.ONE);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBoolean("param_name", true);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setByte("param_name", (byte) 6);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBytes("param_name", "bytes".getBytes());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBytes(1, "bytes".getBytes());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setDate("param_name", Date.valueOf("2019-07-07"));
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setDouble("param_name", 3.0);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );


        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setFloat("param_name", 3.0f);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );


        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setInt("param_name", 3);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setLong("param_name", 3L);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setNString("param_name", "test");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setNull("param_name", Types.NULL);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setNull("param_name", Types.NULL, "null");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setObject("param_name", new Object());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setObject("param_name", new Object(), Types.JAVA_OBJECT);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setObject("param_name", new Object(), Types.JAVA_OBJECT, 2);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setShort("param_name", (short) 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setString("param_name", "test");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setTime("param_name", new Time(50));
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setTimestamp("param_name", new Timestamp(50), Calendar.getInstance());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setTimestamp(
                            "param_name", new Timestamp(50), Calendar.getInstance());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setTimestamp("param_name", new Timestamp(50));
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setURL(1, new URL("http://localhost:8888"));
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setAsciiStream("param_name", new FakeInputStream());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setAsciiStream("param_name", new FakeInputStream(), 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setAsciiStream("param_name", new FakeInputStream(), 5213L);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBinaryStream("param_name", new FakeInputStream());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBinaryStream("param_name", new FakeInputStream(), 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBinaryStream("param_name", new FakeInputStream(), 5213L);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBlob("param_name", new FakeInputStream());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBlob("param_name", new FakeInputStream(), 1L);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setBlob("param_name", new ServiceBlob());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setCharacterStream("param_name", new FakeReader());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setCharacterStream("param_name", new FakeReader(), 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setCharacterStream("param_name", new FakeReader(), 1L);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setClob("param_name", new FakeReader());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setClob("param_name", new FakeReader(), 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setClob("param_name", new FakeNClob());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setClob("param_name", new FakeNClob());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setDate(
                            "param_name", Date.valueOf("2019-07-07"), Calendar.getInstance());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getTime("param_name", Calendar.getInstance());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getTime(1, Calendar.getInstance());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getObject("param_name", String.class);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getObject("param_name", new HashMap<>());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getDate(1, Calendar.getInstance());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.getDate("param_name", Calendar.getInstance());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setRowId("param_name", new FakeRowId());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setSQLXML("param_name", new FakeSQLXML());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    cstm2.setSQLXML("param_name", new FakeSQLXML());
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );
    }

    @AfterAll
    public void tearDown() throws SQLException{
        Assertions.assertFalse(callableStatement.isClosed());
        closeSQLObjects(statement, conn, callableStatement);
    }

}
