package com.fanruan;

import com.fanruan.cache.BeanCacheImpl;
import com.fanruan.handler.DispatcherImpl;
import com.fanruan.pojo.message.RpcRequest;
import com.fanruan.service.jdbc.*;
import com.fanruan.service.jdbc.resultset.ServiceResultSet;
import com.fanruan.service.jdbc.statement.ServiceCallableStatement;
import com.fanruan.service.jdbc.statement.ServicePreparedStatement;
import com.fanruan.service.jdbc.statement.ServiceStatement;
import org.hsqldb.jdbc.*;
import org.hsqldb.types.Type;
import org.junit.jupiter.api.*;

import java.io.*;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.*;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Yichen Dai
 * @date 2022/9/7 10:58
 */
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class ResultSetTest extends BaseJDBCTest{

    private Connection connection = null;
    ResultSet resultSet = null;

    @BeforeAll
    public void setUp() throws SQLException {
        openSocket();
        connection = getConnection();

    }

    @Test
    public void testDate() throws SQLException{
        Statement statement = connection.createStatement();
        statement.execute("create table DemoTable\n" +
                "   (\n" +
                "   Id int NOT NULL AUTO_INCREMENT PRIMARY KEY,\n" +
                "   TestTime TIME, \n" +
                "   TestDate DATE, \n" +
                "   TestStamp TIMESTAMP\n" +
                "   );");

        statement.executeUpdate("insert into DEMOTABLE(TESTTIME,TESTDATE,TESTSTAMP) " +
                "values('15:50:37', '2022-09-07', '2022-09-07 15:50:37');");

        this.resultSet = statement.executeQuery("select TestTime, TestDate, TestStamp from DemoTable;");

        while(resultSet.next()){
            Assertions.assertEquals("15:50:37", resultSet.getTime(1).toString());
            Assertions.assertEquals("2022-09-07", resultSet.getDate(2).toString());
            Assertions.assertEquals("2022-09-07 15:50:37.0", resultSet.getTimestamp(3).toString());
        }

        Assertions.assertFalse(resultSet.isClosed());
    }

    @Test
    public void testGet() throws SQLException, IOException {
        Statement statement = connection.createStatement();

        statement.execute("create table TestTable " +
                "   (" +
                "   testString varchar(20), " +
                "   testBoolean BOOLEAN, " +
                "   testByte NUMERIC(3), " +
                "   testShort NUMERIC(5), " +
                "   testInt INTEGER, " +
                "   testLong BIGINT, " +
                "   testFloat FLOAT, " +
                "   testDouble DOUBLE, " +
                "   testDecimal DECIMAL," +
                "   testBlob BLOB(1024)," +
                "   testClob varchar(100));"
        );

        PreparedStatement prepareStatement = connection.prepareStatement("INSERT INTO TestTable VALUES( " +
                "?, " +
                "?," +
                "?," +
                "?," +
                "?," +
                "?," +
                "?, " +
                "?, " +
                "?," +
                "?," +
                "?);");

        prepareStatement.setString(1, "testString");
        prepareStatement.setBoolean(2, true);
        prepareStatement.setByte(3, Byte.MAX_VALUE);
        prepareStatement.setShort(4, Short.MAX_VALUE);
        prepareStatement.setInt(5, Integer.MAX_VALUE);
        prepareStatement.setLong(6, Long.MAX_VALUE);
        prepareStatement.setFloat(7, 0.2f);
        prepareStatement.setDouble(8, 0.2D);
        prepareStatement.setBigDecimal(9, new BigDecimal(1000));

        byte[] chuck = {
                (byte)0x0f, (byte)0x00, (byte)0x00,
                (byte)0x00, (byte)0x00, (byte)0x00
        };

        Blob blob = new JDBCBlob(chuck);

        prepareStatement.setBlob(10, blob);
        prepareStatement.setClob(11, new JDBCClob("abcd"));
        prepareStatement.executeUpdate();

        ResultSet resultSet = statement.executeQuery("SELECT * FROM TestTable;");

        resultSet.next();
        Assertions.assertEquals("testString", resultSet.getString(1));
        Assertions.assertEquals("testString", resultSet.getString("testString"));
        Assertions.assertEquals(true, resultSet.getBoolean(2));
        Assertions.assertEquals(true, resultSet.getBoolean("testBoolean"));
        Assertions.assertEquals(Byte.MAX_VALUE, resultSet.getByte(3));
        Assertions.assertEquals(Byte.MAX_VALUE, resultSet.getByte("testByte"));
        Assertions.assertEquals(Short.MAX_VALUE, resultSet.getShort(4));
        Assertions.assertEquals(Short.MAX_VALUE, resultSet.getShort("testShort"));
        Assertions.assertEquals(Integer.MAX_VALUE, resultSet.getInt(5));
        Assertions.assertEquals(Integer.MAX_VALUE, resultSet.getInt("testInt"));
        Assertions.assertEquals(Long.MAX_VALUE, resultSet.getLong(6));
        Assertions.assertEquals(Long.MAX_VALUE, resultSet.getLong("testLong"));
        Assertions.assertEquals(0.2f, resultSet.getFloat(7));
        Assertions.assertEquals(0.2f, resultSet.getFloat("testFloat"));
        Assertions.assertEquals(0.2d, resultSet.getDouble(8));
        Assertions.assertEquals(0.2d, resultSet.getDouble("testDouble"));

        Assertions.assertEquals(1000, resultSet.getBigDecimal(9).intValue());
        Assertions.assertEquals(1000, resultSet.getBigDecimal("testDecimal").intValue());


        Assertions.assertEquals(chuck.length, resultSet.getBytes(10).length);
        Assertions.assertEquals(chuck.length, resultSet.getBytes("testBlob").length);

        InputStream in = null;
        try {
            in = resultSet.getBinaryStream("testBlob");
            Assertions.assertEquals(chuck[0], in.read());
        }finally {
            in.close();
        }

        try {
            in = resultSet.getBinaryStream(10);
            Assertions.assertEquals(chuck[0], in.read());
        }finally {
            in.close();
        }

        try {
            in = resultSet.getAsciiStream(11);
            Assertions.assertEquals('a', in.read());
        }finally {
            in.close();
        }

        try {
            in = resultSet.getAsciiStream("testClob");
            Assertions.assertEquals('a', in.read());
        }finally {
            in.close();
        }

        try {
            in = resultSet.getAsciiStream("testClob");
            Assertions.assertEquals('a', in.read());
        }finally {
            in.close();
        }

        Reader reader = null;

        try {
            reader = resultSet.getCharacterStream(11);
            Assertions.assertEquals('a', reader.read());
        }finally {
            reader.close();
        }

        try {
            reader = resultSet.getCharacterStream("testClob");
            Assertions.assertEquals('a', reader.read());
        }finally {
            reader.close();
        }

        Assertions.assertNull(resultSet.getCursorName());
        Assertions.assertNotNull(resultSet.getMetaData());

        closeSQLObjects(statement, prepareStatement);
    }

    @Test
    public void testWrap() throws SQLException {
        String SQL_INSERT = "INSERT INTO EMPLOYEE VALUES 'Tim';";

        String SQL_CREATE = "CREATE TABLE EMPLOYEE (NAME varchar(100));";

        Statement statement = connection.createStatement();

        statement.execute("DROP TABLE IF EXISTS EMPLOYEE;");
        statement.execute(SQL_CREATE);
        statement.executeUpdate(SQL_INSERT);
        ResultSet resultSet = statement.executeQuery("select * from EMPLOYEE");

        Assertions.assertThrows(SQLException.class, () -> resultSet.unwrap(null));
        Assertions.assertFalse(resultSet.isWrapperFor(null));
    }

    @Test
    public void testBlob() throws SQLException, IOException {
        Statement statement = connection.createStatement();

        statement.execute("create table TestBlob" +
                "   (testBlob BLOB(1024));"
        );

        PreparedStatement prepareStatement = connection.prepareStatement("INSERT INTO TestBlob VALUES(?);");
        byte[] chuck = {
                (byte)0x0f, (byte)0x00, (byte)0x00,
                (byte)0x00, (byte)0x00, (byte)0x00
        };

        Blob blob1 = new JDBCBlob(chuck);
        prepareStatement.setBlob(1, blob1);
        prepareStatement.executeUpdate();

        ResultSet resultSet = statement.executeQuery("SELECT * FROM TestBlob");
        resultSet.next();
        Blob blob2 = resultSet.getBlob(1);

        Assertions.assertEquals(blob1.length(), blob2.length());
        Assertions.assertEquals(blob1.getBytes(1,1)[0], blob2.getBytes(1,1)[0]);

        InputStream in1 = null;
        InputStream in2 = null;

        try {
            in1 = blob1.getBinaryStream();
            in2 = blob2.getBinaryStream();
            Assertions.assertEquals(in1.read(), in2.read());
        }finally {
            in1.close();
            in2.close();
        }

        try {
            in1 = blob1.getBinaryStream(1, 6);
            in2 = blob2.getBinaryStream(1, 6);
            Assertions.assertEquals(in1.read(), in2.read());
        }finally {
            in1.close();
            in2.close();
        }


        ServiceBlob serviceBlob = (ServiceBlob) blob2;
        String IDtoInvoke = serviceBlob.getID();
        DispatcherImpl dispatcher = agent.dispatcherImpl;
        BeanCacheImpl beanCache = dispatcher.CACHE.getBeanCache(dbName);
        ServiceBlob blob3 = (ServiceBlob) ProxyFactoryIT.getProxy(ServiceBlob.class, map);

        Assertions.assertThrows(SQLFeatureNotSupportedException.class, () -> {
            blob3.position(chuck, 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLFeatureNotSupportedException.class, () -> {
                    blob3.position(blob1, 1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    blob3.setBytes(1, chuck);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    blob3.setBytes(1, chuck, 1, 6);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        blob2.truncate(6);
        blob2.free();

        closeSQLObjects(statement, prepareStatement);
    }

    @Test
    public void testClob() throws SQLException, IOException {
        Statement statement = connection.createStatement();

        statement.execute("create table TestClob" +
                "   (testClob varchar(20));"
        );

        PreparedStatement prepareStatement = connection.prepareStatement("INSERT INTO TestClob VALUES(?);");

        Clob clob1 = new JDBCClob("abcd");

        prepareStatement.setClob(1, clob1);
        prepareStatement.executeUpdate();

        ResultSet resultSet = statement.executeQuery("SELECT * FROM TestClob");
        resultSet.next();
        Clob clob2 = resultSet.getClob(1);

        Assertions.assertEquals(clob1.length(), clob2.length());
        Assertions.assertEquals(clob1.getSubString(1,2), clob1.getSubString(1,2));

        Reader r1 = null;
        Reader r2 = null;
        try{
             r1 = clob1.getCharacterStream();
             r2 = clob2.getCharacterStream();
            Assertions.assertEquals(r1.read(), r2.read());
        }finally {
            r1.close();
            r2.close();
        }

        try{
            r1 = clob1.getCharacterStream(1, 2);
            r2 = clob2.getCharacterStream(1, 2);
            Assertions.assertEquals(r1.read(), r2.read());
        }finally {
            r1.close();
            r2.close();
        }

        try{
            r1 = clob1.getCharacterStream();
            r2 = clob2.getCharacterStream();
            Assertions.assertEquals(r1.read(), r2.read());
        }finally {
            r1.close();
            r2.close();
        }

        try{
            r1 = clob1.getCharacterStream(1, 2);
            r2 = clob2.getCharacterStream(1, 2);
            Assertions.assertEquals(r1.read(), r2.read());
        }finally {
            r1.close();
            r2.close();
        }

        Assertions.assertEquals(clob1.position("abcd", 1), clob2.position("abcd", 1));
        Assertions.assertEquals(clob1.position(clob1, 1), clob2.position(clob1, 1));

        ServiceClob serviceClob = (ServiceClob) clob2;
        String IDtoInvoke = serviceClob.getID();
        DispatcherImpl dispatcher = agent.dispatcherImpl;
        BeanCacheImpl beanCache = dispatcher.CACHE.getBeanCache(dbName);
        ServiceClob clob3 = (ServiceClob) ProxyFactoryIT.getProxy(ServiceClob.class, map);

        Assertions.assertThrows(SQLException.class, () -> {
                    clob3.setString(1, "");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    clob3.setString(1, "chuck", 1, 6);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    clob3.setAsciiStream(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
                    clob3.setCharacterStream(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );
        closeSQLObjects(statement, prepareStatement);
    }

    @Test
    public void testArray() throws SQLException {
        Connection connection = getConnection();
        Statement statement = connection.createStatement();

        statement.execute("create table TestArray" +
                "   (testArray DOUBLE ARRAY);"
        );

        statement.executeUpdate("INSERT INTO TestArray VALUES ARRAY[3.45, 23.64, 14.01]");

        ResultSet resultSet = statement.executeQuery("SELECT * FROM TestArray");
        resultSet.next();
        Array array = resultSet.getArray(1);

        Assertions.assertEquals("DOUBLE", array.getBaseTypeName());
        Assertions.assertEquals(Types.DOUBLE, array.getBaseType());
        Map<String, Class<?>> map = new HashMap<>();
        map.put("DOUBLE", Double.class);
        Object o1 = array.getArray();
        Object[] arr1 = (Object[]) o1;
        Assertions.assertEquals(3.45, arr1[0]);
        Object o2 = array.getArray(map);
        Object[] arr2 = (Object[]) o2;
        Assertions.assertEquals(3.45, arr2[0]);
        Object o3 = array.getArray(1, 1);
        Object[] arr3 = (Object[]) o3;
        Assertions.assertEquals(3.45, arr3[0]);
        Object o4 = array.getArray(1, 1, map);
        Object[] arr4 = (Object[]) o4;
        Assertions.assertEquals(3.45, arr4[0]);

        ResultSet resultSet1 = array.getResultSet();
        Assertions.assertEquals(3, getSizeOfResultSet(resultSet1));

        ResultSet resultSet2 = array.getResultSet(map);
        Assertions.assertEquals(3, getSizeOfResultSet(resultSet2));

        ResultSet resultSet3 = array.getResultSet(1, 1);
        Assertions.assertEquals(1, getSizeOfResultSet(resultSet3));

        ResultSet resultSet4 = array.getResultSet(1, 1, map);
        Assertions.assertEquals(1, getSizeOfResultSet(resultSet4));
        array.free();
    }

    @Test
    @Disabled("hsql not support struct")
    public void testStruct() throws SQLException {
    }

    @Test
    public void testNClob() throws SQLException, IOException {
        Statement statement = connection.createStatement();

        statement.execute("create table TestNClob" +
                "   (testNClob varchar(20));"
        );

        PreparedStatement prepareStatement = connection.prepareStatement("INSERT INTO TestNClob VALUES(?);");

        NClob nClob1 = new JDBCNClob("abcd");

        prepareStatement.setNClob(1, nClob1);
        prepareStatement.executeUpdate();

        ResultSet resultSet = statement.executeQuery("SELECT * FROM TestNClob");
        resultSet.next();
        NClob nClob2 = resultSet.getNClob(1);

        Assertions.assertEquals(nClob1.length(), nClob2.length());
        Assertions.assertEquals(nClob1.getSubString(1,2), nClob2.getSubString(1,2));

        Reader r1 = null;
        Reader r2 = null;
        try{
            r1 = nClob1.getCharacterStream();
            r2 = nClob2.getCharacterStream();
            Assertions.assertEquals(r1.read(), r2.read());
        }finally {
            r1.close();
            r2.close();
        }

        try{
            r1 = nClob1.getCharacterStream(1, 2);
            r2 = nClob2.getCharacterStream(1, 2);
            Assertions.assertEquals(r1.read(), r2.read());
        }finally {
            r1.close();
            r2.close();
        }

        Assertions.assertEquals(nClob1.position("abcd", 1), nClob2.position("abcd", 1));
        Assertions.assertEquals(nClob1.position(nClob1, 1), nClob2.position(nClob1, 1));

        ServiceNClob serviceNClob = (ServiceNClob) nClob2;
        String IDtoInvoke = serviceNClob.getID();
        DispatcherImpl dispatcher = agent.dispatcherImpl;
        BeanCacheImpl beanCache = dispatcher.CACHE.getBeanCache(dbName);
        ServiceNClob nClob3 = (ServiceNClob) ProxyFactoryIT.getProxy(ServiceNClob.class, map);

        Assertions.assertThrows(SQLException.class, () -> {
            nClob3.setString(1, "");
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
            nClob3.setString(1, "chuck", 1, 6);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
            nClob3.setAsciiStream(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );

        Assertions.assertThrows(SQLException.class, () -> {
            nClob3.setCharacterStream(1);
                    RpcRequest request = map.get(null);
                    request.setIDToInvoke(IDtoInvoke);
                    dispatcher.invokeAsRequest(
                            request,
                            beanCache);
                }
        );
        closeSQLObjects(statement, prepareStatement);
    }

    @Test
    @Disabled("test arg without no-arg constructor Throw SQLException")
    void testUpdatableStream() throws SQLException{
        Statement statement = connection.createStatement();

        statement.executeUpdate("DROP TABLE IF EXISTS updateStreamTest");
        statement.executeUpdate("CREATE TABLE updateStreamTest (keyField INT NOT NULL AUTO_INCREMENT PRIMARY KEY, field1 BLOB)");

        int streamLength = 1024;
        byte[] streamData = new byte[streamLength];

        /* create an updatable statement */
        Statement updStmt = connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);

        /* fill the resultset with some values */
        ResultSet updRs = updStmt.executeQuery("SELECT * FROM updateStreamTest");

        /* move to insertRow */
        updRs.moveToInsertRow();

        /* update the table */
        updRs.updateBinaryStream("field1", new ByteArrayInputStream(streamData), streamLength);

        updRs.insertRow();
    }

    @Test
    public void testLargeBlob() throws SQLException, IOException {
        Statement statement = connection.createStatement();

        statement.execute("create table TestLargeBlob" +
                "   (testBlob BLOB(10240));"
        );

        PreparedStatement prepareStatement = connection.prepareStatement("INSERT INTO TestLargeBlob VALUES(?);");
        byte[] chuck = new byte[10240];

        Blob blob1 = new JDBCBlob(chuck);
        prepareStatement.setBlob(1, blob1);
        prepareStatement.executeUpdate();

        ResultSet resultSet = statement.executeQuery("SELECT * FROM TestLargeBlob");
        resultSet.next();

        closeSQLObjects(statement, prepareStatement);
    }

    @AfterAll
    void tearDown() throws SQLException{
        closeSQLObjects(connection, resultSet);
    }
}
