package com.fanruan;

import org.junit.jupiter.api.*;

import java.io.ByteArrayInputStream;
import java.sql.*;

/**
 * @author Yichen Dai
 * @date 2022/9/13 16:31
 */
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class SavePointTest extends BaseJDBCTest{
    Connection connection = null;

    @BeforeAll
    public void setUp() throws SQLException {
        openSocket();
        this.connection = getConnection();
    }

//    @Test
//    void testSavePoint() throws SQLException{
//        Statement statement = connection.createStatement();
//        connection.setAutoCommit(false);
//        Assertions.assertThrows(SQLException.class, () -> connection.setSavepoint());
//        Assertions.assertThrows(SQLException.class, () -> connection.setSavepoint(""));
//        Assertions.assertThrows(SQLException.class, () -> connection.rollback());
//        Assertions.assertThrows(SQLException.class, () -> connection.rollback(new FakeSavepoint()));
//        Assertions.assertThrows(SQLException.class, () -> connection.releaseSavepoint(new FakeSavepoint()));
//    }

    @Test
    void testSavePoint1() throws SQLException{
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.execute("DROP TABLE if exists testSP;");
        statement.execute("CREATE TABLE testSP (id int);");
        connection.commit();

        statement.executeUpdate("INSERT INTO testSP VALUES (1),(2);");
        ResultSet rs1 = statement.executeQuery("SELECT * FROM testSP;");
        Assertions.assertEquals(2, getSizeOfResultSet(rs1));

        Savepoint savepoint = connection.setSavepoint("test");

        statement.executeUpdate("DELETE FROM testSP WHERE id = 1;");
        ResultSet rs2 = statement.executeQuery("SELECT * FROM testSP;");
        Assertions.assertEquals(1, getSizeOfResultSet(rs2));

        connection.rollback(savepoint);

        ResultSet rs3 = statement.executeQuery("SELECT * FROM testSP;");
        Assertions.assertEquals(2, getSizeOfResultSet(rs3));

        connection.commit();
    }

    @Test
    void testSavePoin2t() throws SQLException{
        connection.setAutoCommit(false);
        Statement statement = connection.createStatement();
        statement.execute("DROP TABLE if exists testSP;");
        statement.execute("CREATE TABLE testSP (id int);");
        connection.commit();

        statement.executeUpdate("INSERT INTO testSP VALUES (1),(2);");
        ResultSet rs1 = statement.executeQuery("SELECT * FROM testSP;");
        Assertions.assertEquals(2, getSizeOfResultSet(rs1));

        statement.executeUpdate("DELETE FROM testSP WHERE id = 1;");
        ResultSet rs2 = statement.executeQuery("SELECT * FROM testSP;");
        Assertions.assertEquals(1, getSizeOfResultSet(rs2));

        connection.rollback();

        ResultSet rs3 = statement.executeQuery("SELECT * FROM testSP;");
        Assertions.assertEquals(0, getSizeOfResultSet(rs3));

        Savepoint savepoint = connection.setSavepoint();
        connection.releaseSavepoint(savepoint);

        connection.commit();

        closeSQLObjects(statement);
    }
}
