package com.fanruan;

import org.junit.jupiter.api.*;

import java.sql.*;


/**
 * @author Yichen Dai
 * @date 2022/8/18 15:27
 */

@TestInstance(TestInstance.Lifecycle.PER_CLASS)
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
@DisplayName("jdbc操作")
public class TestUtil extends BaseJDBCTest{

    private Connection conn = null;
    private Statement st = null;
    private PreparedStatement pst = null;


    @BeforeAll
    void setup(){
        openSocket();
    }

    @Test
    @Order(1)
    void testConnect() throws SQLException {
        conn = getConnection();
    }

    @Test
    @Order(2)
    void testCreateTable1() throws SQLException {
        // 创建 statement
        st = conn.createStatement();

        // 创建表
        int num = st.executeUpdate("DROP TABLE student IF EXISTS;");

        Assertions.assertEquals(0, num);


        num = st.executeUpdate("CREATE TABLE student (" +
                "student_id INTEGER GENERATED BY DEFAULT AS IDENTITY " +
                "(START WITH 1, INCREMENT BY 1) NOT NULL," +
                "student_name VARCHAR(100) NOT NULL," +
                "student_address VARCHAR(100) NOT NULL," +
                "PRIMARY KEY (student_id)" +
                ");");

        Assertions.assertEquals(0, num);
    }

    @Test
    @Order(3)
    void testCreateTable2() throws SQLException {
        // 创建表
        int num = st.executeUpdate("DROP TABLE student_score IF EXISTS;");

        Assertions.assertEquals(0, num);


        num = st.executeUpdate("CREATE TABLE score (" +
                "student_id int(10) PRIMARY KEY NOT NULL," +
                "score int(10) NOT NULL" +
                ");"
        );

        Assertions.assertEquals(0, num);
    }

    @Test
    @Order(4)
    void testInsert1() throws SQLException {
        // 插入数据
        int num = st.executeUpdate("INSERT INTO student VALUES" +
                "(1, '张三', '上海')," +
                "(2, '李四', '北京')," +
                "(3, '王五', '成都');");

        Assertions.assertEquals(3, num);
    }

    @Test
    @Order(5)
    void testInsert2() throws SQLException {
        // 插入数据
        int num = st.executeUpdate("INSERT INTO score VALUES" +
                "(1, 645)," +
                "(2, 627)," +
                "(3, 591);");

        Assertions.assertEquals(3, num);
    }

    @Test
    @Order(6)
    void testUpdate() throws SQLException {
        // 预查询语句 删除指定 ID
        pst = conn.prepareStatement("UPDATE student" +
                " SET student_name = '李华', student_address = '杭州'"+
                "WHERE student_id = ?");

        Assertions.assertNotNull(pst);

        pst.setInt(1, 1);

        int num = pst.executeUpdate();

        Assertions.assertEquals(1, num);
    }

    @Test
    @Order(7)
    void testDelete() throws SQLException {
        // 预查询语句 删除指定 ID
        pst = conn.prepareStatement("delete from student where student_id = ?");

        Assertions.assertNotNull(pst);

        pst.setInt(1, 3);

        int num = pst.executeUpdate();

        Assertions.assertEquals(1, num);
    }

    @Test
    @Order(8)
    void testSelect() throws SQLException {
        ResultSet rs = st.executeQuery("select * from student;");
        Assertions.assertEquals(2, getSizeOfResultSet(rs));
        closeSQLObjects(rs);
    }

    @Test
    @Order(9)
    void testSubSelect() throws SQLException {
        // 插入数据
        ResultSet rs = st.executeQuery(
                "SELECT student_name FROM student " +
                "WHERE student_id IN " +
                "(SELECT student_id " +
                "FROM score " +
                "WHERE score > 600);"
        );
        Assertions.assertEquals(2, getSizeOfResultSet(rs));
        closeSQLObjects(rs);
    }

    @Test
    @Order(10)
    void testJoin() throws SQLException {
        // 插入数据
        ResultSet rs = st.executeQuery(
                "SELECT A.student_name " +
                        "FROM student A JOIN score B " +
                        "ON A.student_id = B.student_id " +
                        "WHERE score > 600;"
        );

        Assertions.assertEquals(2, getSizeOfResultSet(rs));
        closeSQLObjects(rs);
    }

    @Test
    @Order(10)
    @Disabled
    void testErrorQuery() throws SQLException {
        // 插入数据
        ResultSet rs = st.executeQuery("SELECT * FROM TEACHER;");
        Assertions.assertFalse(rs.next());
        closeSQLObjects(rs);
    }

    @AfterAll
    void closeObject() throws SQLException {
        closeSQLObjects(conn, st, pst);
    }
}

